"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));

// src/index.ts
var import_core5 = require("@jscpd/core");
var import_safe6 = require("colors/safe");
var import_finder5 = require("@jscpd/finder");

// src/init/cli.ts
var import_commander = require("commander");
var import_core = require("@jscpd/core");
function initCli(packageJson, argv) {
  const cli = new import_commander.Command(packageJson.name);
  cli.version(packageJson.version).usage("[options] <path ...>").description(packageJson.description).option(
    "-l, --min-lines [number]",
    "min size of duplication in code lines (Default is " + (0, import_core.getOption)("minLines") + ")"
  ).option(
    "-k, --min-tokens [number]",
    "min size of duplication in code tokens (Default is " + (0, import_core.getOption)("minTokens") + ")"
  ).option("-x, --max-lines [number]", "max size of source in lines (Default is " + (0, import_core.getOption)("maxLines") + ")").option(
    "-z, --max-size [string]",
    "max size of source in bytes, examples: 1kb, 1mb, 120kb (Default is " + (0, import_core.getOption)("maxSize") + ")"
  ).option(
    "-t, --threshold [number]",
    "threshold for duplication, in case duplications >= threshold jscpd will exit with error"
  ).option("-c, --config [string]", "path to config file (Default is .jscpd.json in <path>)").option("-i, --ignore [string]", "glob pattern for files what should be excluded from duplication detection").option("--ignore-pattern [string]", "Ignore code blocks matching the regexp patterns").option(
    "-r, --reporters [string]",
    "reporters or list of reporters separated with comma to use (Default is time,console)"
  ).option("-o, --output [string]", "reporters to use (Default is ./report/)").option(
    "-m, --mode [string]",
    'mode of quality of search, can be "strict", "mild" and "weak" (Default is "' + (0, import_core.getOption)("mode") + '")'
  ).option("-f, --format [string]", "format or formats separated by comma (Example php,javascript,python)").option("-p, --pattern [string]", "glob pattern to file search (Example **/*.txt)").option("-b, --blame", "blame authors of duplications (get information about authors from git)").option("-s, --silent", "do not write detection progress and result to a console").option("--store [string]", "use for define custom store (e.g. --store leveldb used for big codebase)").option("-a, --absolute", "use absolute path in reports").option("-n, --noSymlinks", "dont use symlinks for detection in files").option("--ignoreCase", "ignore case of symbols in code (experimental)").option("-g, --gitignore", "ignore all files from .gitignore file").option("--formats-exts [string]", "list of formats with file extensions (javascript:es,es6;dart:dt)").option("-d, --debug", "show debug information, not run detection process(options list and selected files)").option("-v, --verbose", "show full information during detection process").option("--list", "show list of total supported formats").option("--skipLocal", "skip duplicates in local folders, just detect cross folders duplications").option("--exitCode [number]", "exit code to use when code duplications are detected");
  cli.parse(argv);
  return cli;
}

// src/init/options.ts
var import_core3 = require("@jscpd/core");
var import_tokenizer = require("@jscpd/tokenizer");

// src/init/ignore.ts
var import_fs = require("fs");
var gitignoreToGlob = require("gitignore-to-glob");
function initIgnore(options) {
  const ignore = options.ignore || [];
  if (options.gitignore && (0, import_fs.existsSync)(process.cwd() + "/.gitignore")) {
    let gitignorePatterns = gitignoreToGlob(process.cwd() + "/.gitignore") || [];
    gitignorePatterns = gitignorePatterns.map(
      (pattern) => pattern.substr(pattern.length - 1) === "/" ? `${pattern}**/*` : pattern
    );
    ignore.push(...gitignorePatterns);
    ignore.map((pattern) => pattern.replace("!", ""));
  }
  return ignore;
}

// src/options.ts
var import_path = require("path");
var import_fs2 = require("fs");
var import_fs_extra = require("fs-extra");
var import_core2 = require("@jscpd/core");
var import_finder = require("@jscpd/finder");
var convertCliToOptions = (cli) => {
  const result = {
    minTokens: cli.minTokens ? parseInt(cli.minTokens) : void 0,
    minLines: cli.minLines ? parseInt(cli.minLines) : void 0,
    maxLines: cli.maxLines ? parseInt(cli.maxLines) : void 0,
    maxSize: cli.maxSize,
    debug: cli.debug,
    store: cli.store,
    pattern: cli.pattern,
    executionId: cli.executionId,
    silent: cli.silent,
    blame: cli.blame,
    verbose: cli.verbose,
    cache: cli.cache,
    output: cli.output,
    format: cli.format,
    formatsExts: (0, import_finder.parseFormatsExtensions)(cli.formatsExts),
    list: cli.list,
    mode: cli.mode,
    absolute: cli.absolute,
    noSymlinks: cli.noSymlinks,
    skipLocal: cli.skipLocal,
    ignoreCase: cli.ignoreCase,
    gitignore: cli.gitignore,
    exitCode: cli.exitCode
  };
  if (cli.threshold !== void 0) {
    result.threshold = Number(cli.threshold);
  }
  if (cli.reporters) {
    result.reporters = cli.reporters.split(",");
  }
  if (cli.format) {
    result.format = cli.format.split(",");
  }
  if (cli.ignore) {
    result.ignore = cli.ignore.split(",");
  }
  if (cli.ignorePattern) {
    result.ignorePattern = cli.ignorePattern.split(",");
  }
  result.path = cli.path ? [cli.path].concat(cli.args) : cli.args;
  if (result.path.length === 0) {
    delete result.path;
  }
  Object.keys(result).forEach((key) => {
    if (typeof result[key] === "undefined") {
      delete result[key];
    }
  });
  return result;
};
var readConfigJson = (config) => {
  const configFile = config ? (0, import_path.resolve)(config) : (0, import_path.resolve)(".jscpd.json");
  const configExists = (0, import_fs2.existsSync)(configFile);
  if (configExists) {
    const result = { config: configFile, ...(0, import_fs_extra.readJSONSync)(configFile) };
    if (result.path) {
      result.path = result.path.map((path) => (0, import_path.resolve)((0, import_path.dirname)(configFile), path));
    }
    return result;
  }
  return {};
};
var readPackageJsonConfig = () => {
  const config = (0, import_path.resolve)(process.cwd() + "/package.json");
  if ((0, import_fs2.existsSync)(config)) {
    const json = (0, import_fs_extra.readJSONSync)(config);
    if (json.jscpd && json.jscpd.path) {
      json.jscpd.path = json.jscpd.path.map((path) => (0, import_path.resolve)((0, import_path.dirname)(config), path));
    }
    return json.jscpd ? { config, ...json.jscpd } : {};
  }
  return {};
};
function prepareOptions(cli) {
  const storedConfig = readConfigJson(cli.config);
  const packageJsonConfig = readPackageJsonConfig();
  const argsConfig = convertCliToOptions(cli);
  const result = {
    ...(0, import_core2.getDefaultOptions)(),
    ...packageJsonConfig,
    ...storedConfig,
    ...argsConfig
  };
  result.reporters = result.reporters || [];
  result.listeners = result.listeners || [];
  if (result.silent) {
    result.reporters = result.reporters.filter(
      (reporter) => !reporter.includes("console")
    ).concat("silent");
  }
  if (result.threshold !== void 0) {
    result.reporters = [...result.reporters, "threshold"];
  }
  return result;
}

// src/init/options.ts
function initOptionsFromCli(cli) {
  const options = prepareOptions(cli);
  options.format = options.format || (0, import_tokenizer.getSupportedFormats)();
  options.mode = (0, import_core3.getModeHandler)(options.mode);
  options.ignore = initIgnore(options);
  return options;
}

// src/print/files.ts
var import_safe = require("colors/safe");
function printFiles(files) {
  files.forEach((stats) => {
    console.log((0, import_safe.grey)(stats.path));
  });
  console.log((0, import_safe.bold)(`Found ${files.length} files to detect.`));
}

// src/print/options.ts
var import_safe2 = require("colors/safe");
function printOptions(options) {
  console.log((0, import_safe2.bold)((0, import_safe2.white)("Options:")));
  console.dir(options);
}

// src/print/supported-format.ts
var import_safe3 = require("colors/safe");
var import_tokenizer2 = require("@jscpd/tokenizer");
function printSupportedFormat() {
  console.log((0, import_safe3.bold)((0, import_safe3.white)("Supported formats: ")));
  console.log((0, import_tokenizer2.getSupportedFormats)().join(", "));
  process.exit(0);
}

// src/index.ts
var import_crypto = require("crypto");

// src/init/store.ts
var import_core4 = require("@jscpd/core");
var import_safe4 = require("colors/safe");
function getStore(storeName) {
  if (storeName) {
    const packageName = "@jscpd/" + storeName + "-store";
    try {
      const store = require(packageName).default;
      return new store();
    } catch (e) {
      console.error((0, import_safe4.red)("store name " + storeName + " not installed."));
    }
  }
  return new import_core4.MemoryStore();
}

// src/index.ts
var import_tokenizer3 = require("@jscpd/tokenizer");

// src/init/reporters.ts
var import_finder2 = require("@jscpd/finder");
var import_safe5 = require("colors/safe");
var import_html_reporter = __toESM(require("@jscpd/html-reporter"));
var import_jscpd_sarif_reporter = __toESM(require("jscpd-sarif-reporter"));
var reporters = {
  xml: import_finder2.XmlReporter,
  json: import_finder2.JsonReporter,
  csv: import_finder2.CSVReporter,
  markdown: import_finder2.MarkdownReporter,
  consoleFull: import_finder2.ConsoleFullReporter,
  html: import_html_reporter.default,
  console: import_finder2.ConsoleReporter,
  silent: import_finder2.SilentReporter,
  threshold: import_finder2.ThresholdReporter,
  xcode: import_finder2.XcodeReporter,
  sarif: import_jscpd_sarif_reporter.default
};
function registerReporters(options, detector) {
  options.reporters.forEach((reporter) => {
    if (reporter in reporters) {
      detector.registerReporter(new reporters[reporter](options));
    } else {
      try {
        const reporterClass = require(`@jscpd/${reporter}-reporter`).default;
        detector.registerReporter(new reporterClass(options));
      } catch (e) {
        try {
          const reporterClass = require(`jscpd-${reporter}-reporter`).default;
          detector.registerReporter(new reporterClass(options));
        } catch (e2) {
          console.log((0, import_safe5.yellow)(`warning: ${reporter} not installed (install packages named @jscpd/${reporter}-reporter or jscpd-${reporter}-reporter)`));
          console.log((0, import_safe5.grey)(e2.message));
        }
      }
    }
  });
}

// src/init/subscribers.ts
var import_finder3 = require("@jscpd/finder");
function registerSubscribers(options, detector) {
  if (options.verbose) {
    detector.registerSubscriber(new import_finder3.VerboseSubscriber(options));
  }
  if (!options.silent) {
    detector.registerSubscriber(new import_finder3.ProgressSubscriber(options));
  }
}

// src/init/hooks.ts
var import_finder4 = require("@jscpd/finder");
function registerHooks(options, detector) {
  detector.registerHook(new import_finder4.FragmentsHook());
  if (options.blame) {
    detector.registerHook(new import_finder4.BlamerHook());
  }
}

// src/index.ts
var import_fs_extra2 = require("fs-extra");
var TIMER_LABEL = "Detection time:";
var detectClones = (opts, store = void 0) => {
  const options = { ...(0, import_core5.getDefaultOptions)(), ...opts };
  options.format = options.format || (0, import_tokenizer3.getSupportedFormats)();
  const files = (0, import_finder5.getFilesToDetect)(options);
  const hashFunction = (value) => {
    return (0, import_crypto.createHash)("md5").update(value).digest("hex");
  };
  options.hashFunction = options.hashFunction || hashFunction;
  const currentStore = store || getStore(options.store);
  const statistic = new import_core5.Statistic();
  const tokenizer = new import_tokenizer3.Tokenizer();
  const detector = new import_finder5.InFilesDetector(tokenizer, currentStore, statistic, options);
  registerReporters(options, detector);
  registerSubscribers(options, detector);
  registerHooks(options, detector);
  if (!options.silent) {
    console.time((0, import_safe6.italic)((0, import_safe6.grey)(TIMER_LABEL)));
  }
  return detector.detect(files).then((clones) => {
    if (!options.silent) {
      console.timeEnd((0, import_safe6.italic)((0, import_safe6.grey)(TIMER_LABEL)));
    }
    return clones;
  });
};
async function jscpd(argv, exitCallback) {
  const packageJson = (0, import_fs_extra2.readJSONSync)(__dirname + "/../package.json");
  const cli = initCli(packageJson, argv);
  const options = initOptionsFromCli(cli);
  if (options.list) {
    printSupportedFormat();
  }
  if (options.debug) {
    printOptions(options);
  }
  if (!options.path || options.path.length === 0) {
    options.path = [process.cwd()];
  }
  if (options.debug) {
    const files = (0, import_finder5.getFilesToDetect)(options);
    printFiles(files);
    return Promise.resolve([]);
  } else {
    const store = getStore(options.store);
    return detectClones(options, store).then((clones) => {
      if (clones.length > 0) {
        exitCallback?.(options.exitCode || 0);
      }
      return clones;
    }).finally(() => {
      store.close();
    });
  }
}

// bin/jscpd.ts
(async () => {
  try {
    await jscpd(process.argv, process.exit);
  } catch (e) {
    console.log(e);
    process.exit(1);
  }
})();
